#include <stdio.h>
#include "libharkio3.h"
#include <libxml/xmlreader.h>
#include <string.h>

enum atttype {STATE_NONE,STATE_ITEM,STATE_PRICE};
enum checkType {SRC,TSP,IMPULSE,MIC,NOISE};





typedef struct
{
    int hark;
    int  config;
    int  coment;
    int  SynchronousAverage;
    int  TSPpath;
    int  TSPOffset;
    int  PeakSearch;
    int  nfft;
    int  samplingRate;
    int  signalMax;
    int  TSPLength;
    int  positions;
    int  position;
    int neighbors;
    int neighbor ;
    char type_name[12];
} status_t;



//global
status_t status;




void processNode(xmlTextReaderPtr reader );
void status_cls();
int valid_check();
void help(void);




/*--------------------------------------------------*/
//
//help
/*--------------------------------------------------*/
void help(void)
{
	printf("                                                     \n");
	printf("                                                     \n");
	printf("[NAME:]                                              \n");
	printf("  harktoolcli-validxml                               \n");
	printf("                                                     \n");
	printf("[Description:]                                       \n");
	printf("  Consistency check of xml file                      \n");
	printf("                                                     \n");
	printf("[Usage :]                                            \n");
    printf("  harktoolcli-validxml filename(xml)                 \n");
	printf("                                                     \n");
    printf("    positions_type: mic,src,tsp,impulse,noise        \n");
	printf("                                                     \n");
}









/*--------------------------------------------------*/
//
//
/*--------------------------------------------------*/
int main(int argc, char **argv)
{

    int ret;
    xmlTextReaderPtr reader;



    // argc check
    if(argc < 1)
    {
		harkio_Log_printf(harkio_Log_Error , "argument error!!" );
        help();
		exit(EXIT_FAILURE);
    }



    // init
    status_cls();


    //  Readerの作成
    reader = xmlNewTextReaderFilename(argv[1]);

    //  次のノードに移動
    ret = xmlTextReaderRead(reader);
    while (ret == 1)
    {
      //  現在のノードを処理
      processNode(reader);

      //  次のノードに移動
      ret = xmlTextReaderRead(reader);
    }

    //  Reader のすべてのリソースを開放
    xmlFreeTextReader(reader);



	//positions type check
    if( strcmp(status.type_name ,  "source"     ) != 0 &&
        strcmp(status.type_name ,  "tsp"        ) != 0 &&
        strcmp(status.type_name ,  "impulse"    ) != 0 &&
        strcmp(status.type_name ,  "noise"      ) != 0 &&
        strcmp(status.type_name ,  "microphone" ) != 0  )
    {
		printf("Consistency error\n");
		printf("Unknown positions!! [%s]\n" , status.type_name );
		exit(EXIT_FAILURE);
    }


	ret = valid_check();

    exit(ret);
}








/*---------------------------------------------------------*/
//
//
/*---------------------------------------------------------*/
void status_cls()
{
    status.hark = 0;
    status.config = 0;
    status.coment =0;
    status.SynchronousAverage = 0;
    status.TSPpath = 0;
    status.TSPOffset = 0;
    status.PeakSearch = 0;
    status.nfft = 0;
    status.samplingRate = 0;
    status.signalMax = 0;
    status.TSPLength = 0;
    status.positions = 0;
    status.position = 0;
    status.neighbor = 0;
    status.neighbor = 0;
    memset(status.type_name , 0 , 12);
}






/*---------------------------------------------------------*/
//
//
/*---------------------------------------------------------*/
int valid_check( )
{

    char msg[1024];

    memset(msg ,0 , 1024);




    if( strcmp(status.type_name ,  "source"     ) == 0 ||
        strcmp(status.type_name ,  "tsp"        ) == 0 ||
        strcmp(status.type_name ,  "impulse"    ) == 0 )
	{

        if( status.hark   == 0 )
        {
            sprintf(msg ,"%s", "[hark_xml]");
        }

		//config tag check
        if( status.config == 0 )
        {
            strcat(msg ,  (char* )"[config]");
        }

        if( status.coment == 0)
        {
            strcat(msg , "[comment]");

        }
        if( status.SynchronousAverage == 0)
        {
            strcat(msg , "[SynchronousAverage]");

        }
        if( status.TSPpath      == 0)
        {
            strcat(msg , "[TSPpath]");

        }
        if( status.TSPOffset    == 0)
        {
            strcat(msg , "[TSPOffset]");

        }
        if( status.PeakSearch   == 0)
        {
            strcat(msg , "[PeakSearch]");

        }
        if( status.nfft         == 0)
        {
            strcat(msg , "[nfft]");

        }
        if( status.samplingRate == 0)
        {
            strcat(msg , "[samplingRate]");

        }
        if( status.signalMax    == 0)
        {
            strcat(msg , "[signalMax]");

        }
        if( status.TSPLength    == 0)
        {
            strcat(msg , "[TSPLength]");

        }

		//positions tag
        if( status.positions    == 0)
        {
            strcat(msg , "[positions]");

        }

        if( status.position     == 0)
        {
            strcat(msg , "[position]");

        }

		//neighbors tag
        if( status.neighbors    == 0)
        {
            strcat(msg , "[neighbors]");

        }
        if( status.neighbor     == 0)
        {
            strcat(msg , "[neighbor]");
        }
	}



    /*----------------------------------------------------------*/
    /*                                                          */
    /*----------------------------------------------------------*/
    if( strcmp(status.type_name ,  "noise"      ) == 0 ||
        strcmp(status.type_name ,  "microphone" ) == 0 )
	{

        if( status.hark   == 0 )
        {
            sprintf(msg ,"%s", "[hark_xml]");
        }

		//positions tag
        if( status.positions    == 0)
        {
            strcat(msg , "[positions]");

        }

        if( status.position     == 0)
        {
            strcat(msg , "[position]");

        }

		//neighbors tag
        if( status.neighbors    == 0)
        {
            strcat(msg , "[neighbors]");

        }
        if( status.neighbor     == 0)
        {
            strcat(msg , "[neighbor]");
        }

    }



    if( strlen(msg) == 0 )
    {
		printf("SUCCESS\n");
        return(EXIT_SUCCESS);
    }
    else
    {
		printf("Consistency error\n");
        printf("%s\n" , msg);
		return(EXIT_FAILURE);
    }

}





/*---------------------------------------------------------*/
//
//
/*---------------------------------------------------------*/
void processNode(xmlTextReaderPtr reader )
{

    xmlElementType nodeType;
    xmlChar *name ,*name1 , *value1 ;

    //  ノード情報の取得
    nodeType = xmlTextReaderNodeType(reader);       //  ノードタイプ
    name     = xmlTextReaderName(reader);               //  ノード名


    if (nodeType == XML_ELEMENT_NODE) //  開始
    {

        if ( xmlStrcmp(name, BAD_CAST("hark_xml") ) == 0 )
        {
             status.hark = 1;
        }
        if ( xmlStrcmp(name, BAD_CAST("config") ) == 0 )
        {
            status.config = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("comment") ) == 0 )
        {
            status.coment =1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("SynchronousAverage") ) == 0 )
        {
            status.SynchronousAverage = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("TSPpath") ) == 0 )
        {
            status.TSPpath = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("TSPOffset") ) == 0 )
        {
            status.TSPOffset = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("PeakSearch") ) == 0 )
        {
            status.PeakSearch = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("nfft") ) == 0 )
        {
            status.nfft = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("samplingRate") ) == 0 )
        {
            status.samplingRate = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("signalMax") ) == 0 )
        {
            status.signalMax = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("TSPLength") ) == 0 )
        {
            status.TSPLength = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("positions") ) == 0 )
        {
            status.positions = 1;

            do{
                  name1  = xmlTextReaderName(reader);
                  value1 = xmlTextReaderValue(reader);

                  if(xmlStrcmp(name1, BAD_CAST "type") == 0)
                  {
                       strcpy( status.type_name, (char*)value1);
                       xmlFree(name1);
                       xmlFree(value1);
                       break;
                  }
              }while(xmlTextReaderMoveToNextAttribute(reader) == 1);

        }
        else if ( xmlStrcmp(name, BAD_CAST("position") ) == 0 )
        {
            status.position = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("neighbors") ) == 0 )
        {
            status.neighbors = 1;
        }
        else if ( xmlStrcmp(name, BAD_CAST("neighbor") ) == 0 )
        {
            status.neighbor = 1;
        }
    }

    xmlFree(name);

}






